// =======================================================================
// AnalogGrapher.java
// By Patrick Yukman
// For the GreenCube 4 mission at Dartmouth College
// Headed by Kristina Lynch
//
// Purpose and Operation: 
//     This file defines the analog grapher component of the Darkwing
//     DUCKS real time display panel. It loads the last 2000 or so
//     characters out of a specified file, and parses those lines to
//     pull out the data and graph the 3 axes of the magnetometer and
//     accelerometer. This grapher has the capability to pull data from
//     any specified file that is loaded into its file array, and this
//     is used with the button panel to create dynamic, user-selected
//     displaying of data. 
// =======================================================================

import java.io.IOException;
import java.io.File;
import javax.swing.JPanel;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Point;


public class AnalogGrapher extends JPanel {
	static final long serialVersionUID = 0;		// to suppress IDE warnings
	final int LINES = 50;						// size of line array to hold data
	String[] lineArray = new String[LINES];		// initialized line array
	File[] duckFiles;							// file array to maintain pointers to analog data
	File data;									// file pointer to current data file
	int[] ax = new int[LINES];					// Acc X axis data array
	int[] ay = new int[LINES];					// Acc Y axis data array
	int[] az = new int[LINES];					// Acc Z axis data array
	int[] mx = new int[LINES];					// Mag X axis data array
	int[] my = new int[LINES];					// Mag Y axis data array
	int[] mz = new int[LINES];					// Mag Z axis data array
	int numOfDataPoints = 0;					// num of data points, to compute spacing on panel
	final int LEFT_EDGE = 50;					// pixel location of left edge of graph box
	final int GRAPH_W = 540;					// pixel width of graph box
	final int GRAPH_H = 200;					// pixel height of graph box
	final int GRAPH1_Y = 20;					// y location of corner of first graph
	final int GRAPH2_Y = 250;					// y location of corner of second graph
	int currDuck = 0;							// current duck's data to display
	
	
	// Constructor for the analog grapher. Initializes the file array and attempts to open the
	// first file, which by default is A (since currDuck is initialized to 0)
	
	public AnalogGrapher(File[] files) {
		duckFiles = files;
		try {
			openFile();
		}
		catch (IOException e) {}
	}
	
	
	// Overwritten paintComponent method. These actions are performed every time repaint() is used
	
	public void paintComponent(Graphics page) {
		super.paintComponent(page);
		try {
			openFile();
		}
		catch (IOException e) {
			System.err.println(e.getMessage());
		}
		drawGraphs(page);
		drawAccData(page);
		drawMagData(page);
	}
	
	
	// Public method to allow repainting of the AnalogGrapher
	
	public void draw() {
		this.repaint();
	}
	
	
	// Method to return panel size when getPreferredSize() is called in GPS_Manager
	
	public Dimension getPanelSize() {
		return new Dimension(675,475);
	}
	
	
	// This method reads the last 2000 characters from the specified file (which is hopefully
	// fewer lines than LINES), reads to the end of the file while placing all the valid analog
	// data into a String array, and then lets the parse function extract the relevant data
	
	private void openFile() throws IOException {
		String temp;
		int y = 0;    // integer to keep track of array indices through the loop
		eraseData();  // clear the data arrays
		data = duckFiles[currDuck];  // set the current data pointer to the user-specified file
		BufferedRAF filehandler = new BufferedRAF(data, "r");
		long fileLength = data.length() - 1;
		long filePointer = fileLength - 2000;
		if (filePointer < 0) {
			filePointer = 0;
		}
		
		filehandler.seek(filePointer);
		
		// Parse the data, using the same criteria as are found in GPS_Manager
		while ((temp = filehandler.readLine2()) != null) {
			if ((temp != null) && (temp.length() != 0) && (temp.length() >= 61) &&
					(temp.length() <= 80) && (temp.contains("A X: ")) && 
					(temp.contains("M X: ")) && (temp.contains("Z: ")) &&
					(temp.indexOf('A') == 0) && (temp.indexOf('M') == 29) && 
					(!temp.contains("$")) && (!temp.contains("GPGGA"))) {
				lineArray[y] = temp;
				y++;
			}
		}
		filehandler.close();
		parseAnalog();
	}
	
	
	// Method to parse out relevant data from array of data lines. Nearly identical to the
	// parse method found in Squidbee.java, but this one updates the number of data points
	// with each successful iteration of the loop
	
	private void parseAnalog() {
		for (int i = 0; i < lineArray.length && lineArray[i] != null; i++, numOfDataPoints++) {
			if (lineArray[i].substring(5, 6).contentEquals("+")) {
				ax[i] = Integer.parseInt(lineArray[i].substring(6, 10));
			}
			else {
				ax[i] = Integer.parseInt(lineArray[i].substring(5, 10));
			}
			if (lineArray[i].substring(14, 15).contentEquals("+")) {
				ay[i] = Integer.parseInt(lineArray[i].substring(15, 19));
			}
			else {
				ay[i] = Integer.parseInt(lineArray[i].substring(14, 19));
			}
			if (lineArray[i].substring(23, 24).contentEquals("+")) {
				az[i] = Integer.parseInt(lineArray[i].substring(24, 28));
			}
			else {
				az[i] = Integer.parseInt(lineArray[i].substring(23, 28));
			}
			if (lineArray[i].substring(34, 35).contentEquals("+")) {
				mx[i] = Integer.parseInt(lineArray[i].substring(35, 39));
			}
			else {
				mx[i] = Integer.parseInt(lineArray[i].substring(34, 39));
			}
			if (lineArray[i].substring(43, 44).contentEquals("+")) {
				my[i] = Integer.parseInt(lineArray[i].substring(44, 48));
			}
			else {
				my[i] = Integer.parseInt(lineArray[i].substring(43, 48));
			}
			if (lineArray[i].substring(52, 53).contentEquals("+")) {
				mz[i] = Integer.parseInt(lineArray[i].substring(53, 57));
			}
			else {
				mz[i] = Integer.parseInt(lineArray[i].substring(52, 57));
			}
		}
	}
	
	
	// Method to draw the graph backgrounds (labels, zeroing lines, etc.)
	
	public void drawGraphs(Graphics page) {
		page.setColor(Color.white);
		page.fillRect(LEFT_EDGE, GRAPH1_Y, GRAPH_W, GRAPH_H);
		page.fillRect(LEFT_EDGE, GRAPH2_Y, GRAPH_W, GRAPH_H);
		page.setColor(Color.black);
		page.drawRect(LEFT_EDGE, GRAPH1_Y, GRAPH_W, GRAPH_H);
		page.drawRect(LEFT_EDGE, GRAPH2_Y, GRAPH_W, GRAPH_H);
		page.drawLine(LEFT_EDGE, GRAPH1_Y+(GRAPH_H/2), LEFT_EDGE+GRAPH_W, 
				GRAPH1_Y+(GRAPH_H/2));
		page.drawLine(LEFT_EDGE, GRAPH2_Y+(GRAPH_H/2), LEFT_EDGE+GRAPH_W, 
				GRAPH2_Y+(GRAPH_H/2));
		
		page.drawString("-1200",7,450);
		page.drawString("0",37,355);
		page.drawString("1200",15,260);
		page.drawString("-1500",7,220);
		page.drawString("0",37,125);
		page.drawString("1500",15,30);
	}
	
	
	// This method draws all of the accelerometer data in one fell swoop. It computes the spacing
	// required for all the data points to draw properly on the background and within the bounds of
	// the graph.
	
	public void drawAccData(Graphics page) {
		Point previous = new Point(0,0);
		int x,y,pointDistance;
		double Y_RES = 3000/GRAPH_H;
		if (numOfDataPoints > 1) {
			pointDistance = (GRAPH_W / (numOfDataPoints - 1));
			
			// AX Data
			page.setColor(Color.blue);
			for (int i = 0; i < numOfDataPoints; i++) {
				x = LEFT_EDGE+(i*pointDistance)-2;  // data point x coord
				y = (int)Math.round(GRAPH1_Y+(3000-(ax[i]+1500))/(Y_RES))-2;  // data point y coord
				page.fillRect(x, y, 4, 4);  // the data point
				if (i == 0) {
					previous.x = x + 2;
					previous.y = y + 2;
				}
				else {
					page.drawLine(previous.x, previous.y, x+2, y+2);
					previous.x = x + 2;
					previous.y = y + 2;
				}
			}
			page.fillRect(LEFT_EDGE+GRAPH_W+20, GRAPH1_Y+40, 10, 10);
			page.setColor(Color.black);
			page.drawRect(LEFT_EDGE+GRAPH_W+20, GRAPH1_Y+40, 10, 10);
			page.drawString("= AX", LEFT_EDGE+GRAPH_W+35, GRAPH1_Y+50);
			
			// AY Data
			page.setColor(Color.red);
			for (int i = 0; i < numOfDataPoints; i++) {
				x = LEFT_EDGE+(i*pointDistance)-2;
				y = (int)Math.round(GRAPH1_Y+(2400-(ay[i]+1200))/(Y_RES))-2;
				page.fillRect(x, y, 4, 4);
				if (i == 0) {
					previous.x = x + 2;
					previous.y = y + 2;
				}
				else {
					page.drawLine(previous.x, previous.y, x+2, y+2);
					previous.x = x + 2;
					previous.y = y + 2;
				}
			}
			page.fillRect(LEFT_EDGE+GRAPH_W+20, GRAPH1_Y+60, 10, 10);
			page.setColor(Color.black);
			page.drawRect(LEFT_EDGE+GRAPH_W+20, GRAPH1_Y+60, 10, 10);
			page.drawString("= AY", LEFT_EDGE+GRAPH_W+35, GRAPH1_Y+70);
			
			// AZ Data
			page.setColor(Color.black);
			for (int i = 0; i < numOfDataPoints; i++) {
				x = LEFT_EDGE+(i*pointDistance)-2;
				y = (int)Math.round(GRAPH1_Y+(2400-(az[i]+1200))/(Y_RES))-2;
				page.fillRect(x, y, 4, 4);
				if (i == 0) {
					previous.x = x + 2;
					previous.y = y + 2;
				}
				else {
					page.drawLine(previous.x, previous.y, x+2, y+2);
					previous.x = x + 2;
					previous.y = y + 2;
				}
			}
			page.fillRect(LEFT_EDGE+GRAPH_W+20, GRAPH1_Y+80, 10, 10);
			page.setColor(Color.black);
			page.drawRect(LEFT_EDGE+GRAPH_W+20, GRAPH1_Y+80, 10, 10);
			page.drawString("= AZ", LEFT_EDGE+GRAPH_W+35, GRAPH1_Y+90);
		}
	}
	
	
	// This method draws all of the magnetometer data in one fell swoop. It computes the spacing
	// required for all the data points to draw properly on the background and within the bounds of
	// the graph.
	
	public void drawMagData(Graphics page) {
		Point previous = new Point(0,0);
		int x,y;
		double Y_RES = 2400/GRAPH_H;
		if (numOfDataPoints > 1) {
			int pointDistance = (GRAPH_W / (numOfDataPoints - 1));
			
			// MX Data
			page.setColor(Color.cyan);
			for (int i = 0; i < numOfDataPoints; i++) {
				x = LEFT_EDGE+(i*pointDistance)-2;
				y = (int)Math.round(GRAPH2_Y+(2400-(mx[i]+1200))/(Y_RES))-2;
				page.fillRect(x, y, 4, 4);
				if (i == 0) {
					previous.x = x + 2;
					previous.y = y + 2;
				}
				else {
					page.drawLine(previous.x, previous.y, x+2, y+2);
					previous.x = x + 2;
					previous.y = y + 2;
				}
			}
			page.fillRect(LEFT_EDGE+GRAPH_W+20, GRAPH2_Y+40, 10, 10);
			page.setColor(Color.black);
			page.drawRect(LEFT_EDGE+GRAPH_W+20, GRAPH2_Y+40, 10, 10);
			page.drawString("= MX", LEFT_EDGE+GRAPH_W+35, GRAPH2_Y+50);
			
			// MY Data
			page.setColor(Color.magenta);
			for (int i = 0; i < numOfDataPoints; i++) {
				x = LEFT_EDGE+(i*pointDistance)-2;
				y = (int)Math.round(GRAPH2_Y+(2400-(my[i]+1200))/(Y_RES))-2;
				page.fillRect(x, y, 4, 4);
				if (i == 0) {
					previous.x = x + 2;
					previous.y = y + 2;
				}
				else {
					page.drawLine(previous.x, previous.y, x+2, y+2);
					previous.x = x + 2;
					previous.y = y + 2;
				}
			}
			page.fillRect(LEFT_EDGE+GRAPH_W+20, GRAPH2_Y+60, 10, 10);
			page.setColor(Color.black);
			page.drawRect(LEFT_EDGE+GRAPH_W+20, GRAPH2_Y+60, 10, 10);
			page.drawString("= MY", LEFT_EDGE+GRAPH_W+35, GRAPH2_Y+70);
			
			// MZ Data
			page.setColor(Color.green);
			for (int i = 0; i < numOfDataPoints; i++) {
				x = LEFT_EDGE+(i*pointDistance)-2;
				y = (int)Math.round(GRAPH2_Y+(2400-(mz[i]+1200))/(Y_RES))-2;
				page.fillRect(x, y, 4, 4);
				if (i == 0) {
					previous.x = x + 2;
					previous.y = y + 2;
				}
				else {
					page.drawLine(previous.x, previous.y, x+2, y+2);
					previous.x = x + 2;
					previous.y = y + 2;
				}
			}
			page.fillRect(LEFT_EDGE+GRAPH_W+20, GRAPH2_Y+80, 10, 10);
			page.setColor(Color.black);
			page.drawRect(LEFT_EDGE+GRAPH_W+20, GRAPH2_Y+80, 10, 10);
			page.drawString("= MZ", LEFT_EDGE+GRAPH_W+35, GRAPH2_Y+90);
		}
	}
	
	
	// This method wipes the data out of the arrays, allowing for a fresh parse next time one
	// is required or requested
	
	private void eraseData() {
		numOfDataPoints = 0;
		
		for (int i = 0; i < LINES; i++) {
			lineArray[i] = null;
			ax[i] = 0;
			ay[i] = 0;
			az[i] = 0;
			mx[i] = 0;
			my[i] = 0;
			mz[i] = 0;
		}
	}
	
	
	// This method changes the file loaded by the grapher
	
	public void changeFile(int whichDuck) {
		currDuck = whichDuck;
		repaint();
	}
	
	
	// Method to return the int identifier of the current file loaded by the grapher
	
	public int getDuck() {
		return currDuck;
	}
}