// =======================================================================
// GPS_Manager.java
// By Patrick Yukman
// For the GreenCube 4 "Duckey" mission at Dartmouth College
// Headed by Kristina Lynch
//
// Purpose and Operation: This class, called GPS_Manager, actually
//     maintains much more than GPS data. This is the central data hub
//     of the program. In addition to managing the GUI, it does the
//     initial parse of the raw data files, deciding first if a packet
//     is valid data, then handing the sentence off to the appropriate
//     "Squidbee" object for further handling. This is the file that
//     contains the software's infinite while loop. Small duration
//     sleeps are included in the loop to prevent it entering a tight
//     I/O spin, which ends up draining a laptop battery much faster
//     than expected. 
// =======================================================================

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.awt.Color;
import java.io.IOException;
import java.io.File;
import java.io.FileWriter;
import java.io.BufferedWriter;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;


public class GPS_Manager {
	static final long serialVersionUID = 0;   // to suppress warnings in eclipse
	private final int NUM_DUCKS = 20;   // how many ducks are floating? good idea to leave at 20
	private String folderTitle, fileTitle1, fileTitle2, realtimeTitle;   // to manage files
	private final String ducks = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";   // easy way to enumerate ducks
	private Squidbee[] ducklings = new Squidbee[NUM_DUCKS];   // array of Squidbee objects
	private File[] fileArray = new File[NUM_DUCKS];   // array of created files for AnalogGrapher
	File data1, data2;          // raw data files coming in
	private Switchboard hud;    // diagnostic display (numbers and valid check for all ducks)
	AnalogGrapher aDisplay;     // window in which mag/acc data is graphed
	ButtonPanel moreInfo;       // button panel to switch which duck's data is shown in aDisplay
	
	
	// Constructor, which serves the purpose of setting up all the necessary file paths and
	// GUI relationships right at the start
	
	public GPS_Manager() {
		try {
			fileSetup();
		}
		catch (IOException e) {}
	}
	
	
	// Uses file choosers to allow users to find the directories/files to be used for data
	// management. The first chooser prompts for a destination directory. This is the folder that
	// will house the realtime kml file for Google Earth, as well as subdirectories for all the
	// ducks that contain intermediate GPS data and analog data for the grapher. Then the user
	// picks the two data files that are being captured to the hard drive. Once these files
	// are set up and loaded into memory, the parsing can begin. This method also sets up the GUI.
	
	private void fileSetup() throws IOException {
		// First file chooser: pick directory for various files to go into
		String fullFolderPath = "";
		String folder = "";
		JFileChooser chooser = new JFileChooser();
		chooser.setDialogTitle("Please select your destination directory");
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int r = chooser.showOpenDialog(new JFrame());
		if (r == 0) {
			fullFolderPath = chooser.getSelectedFile().getAbsolutePath();
		}
		else {
			System.exit(0);
		}
		// Get date and time to name the folder something unique
		DateFormat dateFormat = new SimpleDateFormat("MM-dd-yyyy_(HHmmss)");
		Date date = new Date();
		folder = "/Realtime_GPS_" + dateFormat.format(date);
		folderTitle = fullFolderPath + folder;
		new File(folderTitle).mkdir();
		for (int i = 0; i < NUM_DUCKS; i++) {
			new File(folderTitle + "/" + ducks.charAt(i)).mkdir();
		}
		
		// Now choose the first of the two data files
		chooser.setDialogTitle("Please select your first incoming data file");
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		r = chooser.showOpenDialog(new JFrame());
		if (r == 0) {
			fileTitle1 = chooser.getSelectedFile().getAbsolutePath();
			data1 = new File(fileTitle1);
		}
		else {
			System.exit(0);
		}
		
		// Choose the second
		chooser.setDialogTitle("Please select your second incoming data file");
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		r = chooser.showOpenDialog(new JFrame());
		if (r == 0) {
			fileTitle2 = chooser.getSelectedFile().getAbsolutePath();
			data2 = new File(fileTitle2);
		}
		else {
			System.exit(0);
		}
		
		// This block writes the top-level kml file, which references all the other duck
		// kml files and tells Google Earth to reload them all at one-second intervals.
		// It also creates NUM_DUCKS Squidbee objects, which are initialized with the
		// filepaths to their respective kml files and analog files, for future handling.
		// The analog files are added to the file array that aDisplay will use to keep
		// track of which file it is reading from.
		realtimeTitle = folderTitle + "/realtime_gps.kml";
		String body1 = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
				"<kml xmlns=\"http://earth.google.com/kml/2.1\">\n" +
				"<Document>\n";
		String duck1 = "<NetworkLink>\n" + "<Link>\n" + "<href>";
		String duck2 = "</href>\n" + "<refreshMode>onInterval</refreshMode>\n" +
				"<refreshInterval>1</refreshInterval>\n" + "</Link>\n" + "</NetworkLink>\n";
		String body2 = "</Document>\n" + "</kml>";
		FileWriter fstream = new FileWriter(realtimeTitle,true);
		BufferedWriter out = new BufferedWriter(fstream);
		out.write(body1);
		String duckKML_Path, duckAnalogPath;
		for (int i = 0; i < NUM_DUCKS; i++) {
			duckKML_Path = folderTitle + "/" + ducks.charAt(i) + "/" + ducks.charAt(i) + ".kml";
			duckAnalogPath = folderTitle + "/" + ducks.charAt(i) + "/" + ducks.charAt(i) + ".txt";
			ducklings[i] = new Squidbee(ducks.charAt(i), duckKML_Path, duckAnalogPath);
			new File(duckKML_Path).createNewFile();
			new File(duckAnalogPath).createNewFile();
			fileArray[i] = new File(duckAnalogPath);
			out.write(duck1 + duckKML_Path + duck2);
		}
		out.write(body2);
		out.close();
		fstream.close();
		
		// Finally, the GUI is set up in this block. All the JPanels are initialized
		// and added to an overseeing JPanel called "primary," which holds them all
		// in place.
		JFrame frame = new JFrame ("Ducks Diagnostic Display");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JPanel primary = new JPanel();
		primary.setBackground(Color.gray);
		hud = new Switchboard(NUM_DUCKS,ducklings,465,633);
		aDisplay = new AnalogGrapher(fileArray);
		moreInfo = new ButtonPanel(100,633,aDisplay);
		primary.add(hud);
		primary.add(moreInfo);
		primary.add(aDisplay);
		frame.getContentPane().add(primary);
		hud.setBackground(Color.gray);
		hud.setPreferredSize(hud.getPanelSize());
		aDisplay.setPreferredSize(aDisplay.getPanelSize());
		frame.pack();
		frame.setVisible(true);
	}
	
	
	// This method is the core of the software. It reads a line from the first data
	// file, makes a decision about its validity as data, and passes the sentence
	// to the associated Squidbee object for further handling, if appropriate. It
	// then does the same with the second file. Repaint is called only when an
	// element of the GUI needs to reflect a change, aside from an automatic 
	// repaint every ~500 ms to update "hud" if no data is being received. The 1 ms
	// sleeps prevent the while loop from running too tightly, which consumes CPU
	// power to a surprising degree. 
	
	public void manageData() throws IOException {
		BufferedRAF filehandler1 = new BufferedRAF(data1, "r"); // see BufferedRAF.java
		BufferedRAF filehandler2 = new BufferedRAF(data2, "r"); // see BufferedRAF.java
		String temp;        // string to read sentence into for a short time
		int idNum;          // to manage Squidbees based on end-of-sentence character ID
		long count = 0;     // to allow an automatic repaint every few run-throughs
		moreInfo.draw();    // paint the button panel this once. No need to repaint often
		
		// the main while loop of the program
		while (true) {
			count++;
			
			// Read a line from the first file. If identified as a GPGGA GPS sentence,
			// pass it to the appropriate Squidbee and update hud. Otherwise, if it's
			// an analog sentence, pass to Squidbee and update hud AND aDisplay IF
			// aDisplay is currently displaying data from the Squidbee in question.
			// If neither GPS or analog, just ignore.
			if ((temp = filehandler1.readLine2()) != null) {
				if ((temp != null) && (temp.length() != 0) && (temp.length() > 59) 
						&& (temp.length() < 80) && (temp.contains("$GPGGA,")) && 
						(temp.indexOf('$') == 0) && (!temp.contains("X")) && 
						(!temp.contains("Y")) && (!temp.contains("Z"))) {
					idNum = ducks.indexOf(getLastChar(temp));
					if ((idNum >= 0) && (idNum < NUM_DUCKS)) {
						ducklings[idNum].manageGPSData(temp);
						hud.draw();
					}
				}
				else if ((temp != null) && (temp.length() != 0) && (temp.length() >= 61) &&
						(temp.length() <= 80) && (temp.contains("A X: ")) && 
						(temp.contains("M X: ")) && (temp.contains("Z: ")) &&
						(temp.indexOf('A') == 0) && (temp.indexOf('M') == 29) && 
						(!temp.contains("$")) && (!temp.contains("GPGGA"))) {
					idNum = ducks.indexOf(getLastChar(temp));
					if ((idNum >= 0) && (idNum < NUM_DUCKS)) {
						ducklings[idNum].manageAnalogData(temp);
						hud.draw();
						if (idNum == aDisplay.getDuck()) {
							aDisplay.draw();
						}
					}
				}
			}
			
			// Sleep 1 ms
			try {
				Thread.sleep(1);
			} catch (InterruptedException e) {}
			
			// Same process as above, but performed with second file.
			if ((temp = filehandler2.readLine2()) != null) {
				if ((temp != null) && (temp.length() != 0) && (temp.length() > 59) 
						&& (temp.length() < 80) && (temp.contains("$GPGGA,")) && 
						(temp.indexOf('$') == 0) && (!temp.contains("X")) && 
						(!temp.contains("Y")) && (!temp.contains("Z"))) {
					idNum = ducks.indexOf(getLastChar(temp));
					if ((idNum >= 0) && (idNum < NUM_DUCKS)) {
						ducklings[idNum].manageGPSData(temp);
						hud.draw();
					}
				}
				else if ((temp != null) && (temp.length() != 0) && (temp.length() >= 61) &&
						(temp.length() <= 80) && (temp.contains("A X: ")) && 
						(temp.contains("M X: ")) && (temp.contains("Z: ")) &&
						(temp.indexOf('A') == 0) && (temp.indexOf('M') == 29) && 
						(!temp.contains("$")) && (!temp.contains("GPGGA"))) {
					idNum = ducks.indexOf(getLastChar(temp));
					if ((idNum >= 0) && (idNum < NUM_DUCKS)) {
						ducklings[idNum].manageAnalogData(temp);
						hud.draw();
						if (idNum == aDisplay.getDuck()) {
							aDisplay.draw();
						}
					}
				}
			}
			
			// Auto-update hud every 250 passes through the loop (~500 ms?)
			if (count % 250 == 0) {
				hud.draw();
			}
			
			// Sleep another 1 ms
			try {
				Thread.sleep(1);
			} catch (InterruptedException e) {}
	
		}
	}
	
	
	// Helper method to return the last character of a string. Used to get the character
	// ID out of a sentence of data.
	
	private char getLastChar(String sentence) {
		return sentence.charAt(sentence.length() - 1);
	}
}