// =======================================================================
// Squidbee.java
// By Patrick Yukman
// For the GreenCube 4 "Duckey" mission at Dartmouth College
// Headed by Kristina Lynch
//
// Purpose and Operation: This class defines a Squidbee object and its
//     relevant methods. Most of the data parsing for the diagnostic
//     display ("hud" in GPS_Manager) is handled by each individual
//     duck's associated Squidbee object. All important KML file parsing
//     and some important analog parsing is handled in this file.
// =======================================================================

import java.util.Scanner;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import java.text.DecimalFormat;


public class Squidbee {
	char name;                                         // name string
	String kmlFile, analogFile;						   // names of req. files
	String coordinates = "";						   // init. coord list
	String color = "";								   // init. color string
	String rawLat, rawLon, numSat, burn;			   // parse storage strs
	double latDeg, lonDeg, latMin, lonMin, lat, lon;   // parse doubles
	DecimalFormat coordForm;						   // decimal format obj
	FileWriter kmlStream, analogStream;				   // req. filewriters
	BufferedWriter kmlOut, analogOut;				   // req. buffwriters
	double recLat, recLon;							   // stored coord memory
	int numSatellites;								   // parsed sat num
	int ax, ay, az, mx, my, mz;						   // acc/mag data storage
	long captureTime;								   // store system time
	
	
	// Constructor for Squidbee object. Initializes name, kmlFile, analogFile, and
	// coordForm fields. Also randomly generates a hex color for the kml file to
	// display the Squidbee path in
	
	public Squidbee(char charID, String kmlPath, String analogPath) {
		name = charID;
		generateHexColor();
		kmlFile = kmlPath;
		analogFile = analogPath;
		coordForm = new DecimalFormat("###.######");
	}
	
	
	// Method to oversee GPS data handling if called from an outside source (like 
	// GPS_Manager, for instance)
	
	public void manageGPSData(String sentence) {
		parseGPS(sentence);
		generateKML();
	}
	
	
	// Method to parse relevant data out of a GPS sentence printed in GPGGA format.
	// Burn string is simply a place to put unwanted pieces of the sentence
	
	private void parseGPS(String data) {
		Scanner parser = new Scanner(data).useDelimiter(",");  // parse scanner
		burn = parser.next();
		burn = parser.next();
		rawLat = parser.next();
		burn = parser.next();
		rawLon = parser.next();
		burn = parser.next();
		burn = parser.next();
		numSat = parser.next();
		burn = burn + "";
		numSatellites = Integer.parseInt(numSat);
		latDeg = Double.parseDouble(rawLat.substring(0, 2));
		
		// As long as the latitude is not exactly zero (that is, as long as the 
		// Squidbee has a GPS position lock), interpret the parsed data and write
		// the coordinates to the necessary lists and fields
		if (latDeg != 0) {
			lonDeg = Double.parseDouble(rawLon.substring(0, 3));
			latMin = Double.parseDouble(rawLat.substring(2));
			latMin = latMin / 60.0;
			lonMin = Double.parseDouble(rawLon.substring(3));
			lonMin = lonMin / 60.0;
			lat = Double.valueOf(coordForm.format(latDeg + latMin));
			lon = Double.valueOf(coordForm.format(lonDeg + lonMin));
			lon*=-1;
			coordinates = coordinates + lon + "," + lat + "\n";
			recLat = lat;
			recLon = lon;
		}
	}
	
	
	// Method to completely rewrite the kml file associated with the Squidbee, thereby
	// updating the coordinates for the overseeing kml file that loads this one. Uses
	// safe closes within try/catch loops to prevent file pointers from getting out of
	// hand
	
	private void generateKML() {
		try {
			kmlStream = new FileWriter(kmlFile,false);
			kmlOut = new BufferedWriter(kmlStream);
		}
		catch (IOException e) {
			try {
				kmlOut.close();
				kmlStream.close();
			} catch (IOException e2) {}
		}
		
		// This string can be changed to allow for different display options. See
		// https://developers.google.com/kml/documentation/ for info
		String body = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + "<kml>\n" +
				"<Document>\n" + "<name>" + name + "</name>\n" + "<Style id=\"thickBlackLine\">\n"+
				"<LineStyle>\n" + "<color>" + color + "</color>\n" + "<width>5</width>\n" + 
				"</LineStyle>\n" + "</Style>\n" + "<Placemark>\n" + "<name>GPS</name>\n" +
				"<visibility>1</visibility>\n" + "<styleUrl>#thickBlackLine</styleUrl>\n" +
				"<LineString>\n" + "<tessellate>1</tessellate>\n" +
				"<altitudeMode>relativeToGround</altitudeMode>\n" + "<coordinates>\n" + 
				coordinates + "</coordinates>\n" + "</LineString>\n" + "</Placemark>\n" +
				"</Document>\n" + "</kml>";
		try {
			kmlOut.write(body);
			kmlOut.close();
			kmlStream.close();
		}
		catch (IOException e) {
			try {
				kmlOut.close();
				kmlStream.close();
			} catch (IOException e2) {}
		}
	}
	
	
	// Method to generate a random hex color for the Google Earth kml file to display
	// the Squidbee's track in
	
	private void generateHexColor() {
		int num1 = (int)(Math.random()*256);
		if (num1 <= 16) { num1+=16; }
		int num2 = (int)(Math.random()*256);
		if (num2 <= 16) { num2+=16; }
		int num3 = (int)(Math.random()*256);
		if (num3 <= 16) { num3+=16; }
		color = "ff" + Integer.toHexString(num1) + Integer.toHexString(num2) + 
				Integer.toHexString(num3);
	}
	
	
	// Method to oversee analog data handling if called from an outside source
	
	public void manageAnalogData(String sentence) {
		parseAnalog(sentence);
		isValid();
	}
	
	
	// Method to parse relevant data out of the analog sentences sent by the Squidbee.
	// These sentences have the form:
	// A X: +0037 Y: -0212 Z: -0991 M X: -0248 Y: +0395 Z: +0120 2506 A
	
	private void parseAnalog(String data) {
		captureTime = System.currentTimeMillis();  // get time for later comparison
		
		// Write the analog sentence to the specific analog data file associated with
		// this Squidbee. This is for later use by AnalogGrapher
		try {
			analogStream = new FileWriter(analogFile,true);
			analogOut = new BufferedWriter(analogStream);
			analogOut.write(data + "\n");
			analogOut.close();
			analogStream.close();
		}
		catch (IOException e) {
			try {
				analogOut.close();
				analogStream.close();
			} catch (IOException e2) {}
		}
		
		// Now parse the important fields out of the sentence for diagnostic checking
		if (data.substring(5, 6).contentEquals("+")) {
			ax = Integer.parseInt(data.substring(6, 10));
		}
		else {
			ax = Integer.parseInt(data.substring(5, 10));
		}
		if (data.substring(14, 15).contentEquals("+")) {
			ay = Integer.parseInt(data.substring(15, 19));
		}
		else {
			ay = Integer.parseInt(data.substring(14, 19));
		}
		if (data.substring(23, 24).contentEquals("+")) {
			az = Integer.parseInt(data.substring(24, 28));
		}
		else {
			az = Integer.parseInt(data.substring(23, 28));
		}
		if (data.substring(34, 35).contentEquals("+")) {
			mx = Integer.parseInt(data.substring(35, 39));
		}
		else {
			mx = Integer.parseInt(data.substring(34, 39));
		}
		if (data.substring(43, 44).contentEquals("+")) {
			my = Integer.parseInt(data.substring(44, 48));
		}
		else {
			my = Integer.parseInt(data.substring(43, 48));
		}
		if (data.substring(52, 53).contentEquals("+")) {
			mz = Integer.parseInt(data.substring(53, 57));
		}
		else {
			mz = Integer.parseInt(data.substring(52, 57));
		}
	}
	
	
	// Method to determine whether a Squidbee's data is valid. The levels for these checks
	// were developed by Amanda Slagle and Patrick Yukman. For data to be valid, the 
	// Squidbee must have: more than one satellite, a GPS position lock, acc/mag data
	// within reasonable ranges, and a data sentence must have been parsed by the Squidbee
	// sometime within the last 10 seconds. This method is called in Switchboard.java when
	// deciding whether to draw the diagnostic squares in red or green. 
	
	public boolean isValid() {
		boolean test1 = numSatellites > 0;
		boolean test2 = recLat != 0.0;
		boolean test3 = recLat != 0.0;
		boolean test4 = az < 0;
		boolean test5 = ax < 750 || ax > -750;
		boolean test6 = ay < 750 || ay > -750;
		boolean test7 = mx < 1200 || mx > -1200;
		boolean test8 = my < 1200 || my > -1200;
		boolean test9 = mz < 1200 || mz > -1200;
		boolean test10 = (int)((System.currentTimeMillis()-captureTime)/1000) <= 10;
		return test1 && test2 && test3 && test4 && test5 && test6 && test7 && 
				test8 && test9 && test10;
	}
}