// =======================================================================
// Switchboard.java
// By Patrick Yukman
// For the GreenCube 4 mission at Dartmouth College
// Headed by Kristina Lynch
//
// Purpose and Operation: This file specifies the Switchboard, which is
//     a visual display of some diagnostic information for all 20 ducks.
//     The visual layout is a table, with each row representing a duck
//     and each column representing some data value from the duck (GPS
//     position, z-axis of accelerometer, etc.) Each duck object is 
//     polled in turn for its data.
// =======================================================================

import javax.swing.JPanel;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Dimension;


public class Switchboard extends JPanel {
	static final long serialVersionUID = 0;			// to suppress IDE warnings
	private final int numOfDucks;					// stored number of ducks, passed in as param
	private final int TOPBUFF = 30, MARGIN = 70;	// pixel locations to place components off of
	private final int WIDTH, HEIGHT;				// pixel w and h, passed in as params
	private final String[] duckNames = {"Axl", "Buddy", "Chuck", "Diana", "Ella", "Freddie",
			"George","Hendrix","Iggy","Janis","Kurt","Lynyrd","Madonna","Norah","Ozzy","Page",
			"Queen","Ricky","Stevie","Tina"};  		// names of all the ducks
	private Squidbee[] ducklings;					// array of Squidbee objects, passed in param
	
	
	// Constructor for Switchboard object. Initializes necessary global variables from the
	// parameters that are passed in.
	
	public Switchboard(int duckNum, Squidbee[] squids, int w, int h) {
		numOfDucks = duckNum;
		ducklings = squids;
		WIDTH = w;
		HEIGHT = h;
	}
	
	
	// Overwritten paintComponent method. These actions are performed every time repaint() is used
	
	public void paintComponent(Graphics page) {
		super.paintComponent(page);
		drawNonData(page);
		drawData(page);
	}
	
	
	// Public method to allow repainting of the Switchboard
	
	public void draw() {
		this.repaint();
	}
	
	
	// Method to return panel size when getPreferredSize() is called in GPS_Manager
	
	public Dimension getPanelSize() {
		return new Dimension(WIDTH,HEIGHT);
	}
	
	
	// This method draws the lines, boxes, and strings that form the readout display of the 
	// switchboard
	
	private void drawNonData(Graphics page) {
		page.setColor(Color.white);
		page.fillRect(0, 0, WIDTH-1, HEIGHT-1);
		page.setColor(Color.black);
		page.drawRect(0, 0, WIDTH-1, HEIGHT-1);
		for (int i = 0; i < numOfDucks; i++) {
			page.drawLine(0, TOPBUFF+(30*i), WIDTH, TOPBUFF+(30*i));
			page.drawString(duckNames[i], 10, TOPBUFF+20+(30*i));
		}
		page.drawString("#Sats", MARGIN+7, 20);
		page.drawString("Lat.", MARGIN+65, 20);
		page.drawString("Lon.", MARGIN+155, 20);
		page.drawString("Acc. Z", MARGIN+270, 20);
		page.drawString("Valid?", MARGIN+330, 20);
	} 
	
	
	// This method draws the data strings and colors the valid box appropriately for each duck.
	
	private void drawData(Graphics page) {
		page.setColor(Color.black);
		
		// For each duck, print: the number of satellites the GPS sees, the most recently 
		// received latitude coordinate, the most recently received longitude coordinate, the
		// value from the duck's z axis of its accelerometer, and draw the box indicating 
		// whether the data is "good" or not (computed in Squidbee.java)
		for (int i = 0; i < numOfDucks; i++) {
			page.drawString(Integer.toString(ducklings[i].numSatellites), MARGIN+20, 
					TOPBUFF+20+(30*i));
			page.drawString(Double.toString(ducklings[i].recLat), MARGIN+65, TOPBUFF+20+(30*i));
			page.drawString(Double.toString(ducklings[i].recLon), MARGIN+155, TOPBUFF+20+(30*i));
			page.drawString(Integer.toString(ducklings[i].az), MARGIN+265, TOPBUFF+20+(30*i));
			if (ducklings[i].isValid()) {
				page.setColor(Color.green);
			}
			else {
				page.setColor(Color.red);
			}
			page.fillRect(MARGIN+340, TOPBUFF+7+(30*i), 15, 15);
			page.setColor(Color.black);
			page.drawRect(MARGIN+340, TOPBUFF+7+(30*i), 15, 15);
		}
	}
}